<?php
error_reporting(0);
defined( '_JEXEC' ) or die( 'Restricted access' );
	/******************************************************************************************************
	*  Written by Jaka Prasnikar (http://prahec.com)
	*  This script parses Icecast&Shoutcast XML stats and displays them in Multi/Single server configuration
	*  Its very easy to use.
	********************************************************************************************************/

	// This class returns array of radio INFO (Loads of Information)
	class webradio {

		private $streams;

		function __construct($streams_array) {
			// On object creation we just save servers array
			$this->streams = $streams_array;
		}
		// Faster access to script with webradio::info($streams_array, $serverID)
		static public function info($streams, $serverID = '') {

			$obj = new self($streams);
			return $obj->getInfo($serverID);

		}

		// Faster access to script with webradio::all($streams_array)
		static public function all($streams, $cacheName = '') {

			$obj = new self($streams);

			if(extension_loaded('apc') && ini_get('apc.enabled')) {
				return $obj->cached($cacheName);
			} else {
				return $obj->totals();
			}

		}

		// A new function to debug WebRadio setup
		// This function is VERY messy. But it shows data for debugging process.
		public static function debug($array, $serverID = '') {

			// Create Object
			$ob = new self($array);
			$start = microtime(true);

			if ($serverID == '') $key = key($array); else $key = $serverID;

			// Output debug details
			$html = 'WebRadio class will now start debugging process... <br /><br />';

			$html .= '<b>...Connecting to the first server in array:</b>
			<button onclick="document.getElementById(\'conn\').style.display = \'block\'; return false;">Show Details</button>
			<br /><div id="conn" style="display: none;"><pre>';


			// Call GetData
			if ($array[$key]['type'] == 'icecast') {
				$data = $ob->connect("http://{$array[$key]['ip']}:{$array[$key]['port']}/admin/stats", "{$array[$key]['user']}:{$array[$key]['pw']}");
			} else {
				$data = $ob->connect("http://{$array[$key]['ip']}:{$array[$key]['port']}/admin.cgi?sid={$array[$key]['sid']}&pass={$array[$key]['pw']}&mode=viewxml");
			}

			$html .= (($data === false) ? '<b style="color: red;">ERROR: Failed to get data from the server!</b>' : '<b style="color: green;">Successfully fetched data</b>') . '</pre></div>';

			// First function
			$html .= '<b>...Getting the data from first server in array: </b>
			<button onclick="document.getElementById(\'first\').style.display = \'block\'; return false;">Show Details</button>
			<br /><div id="first" style="display: none;"><pre style="background: #f9f9f9;">';

			$data1 = $ob->getData($array[$key]['ip'], $array[$key]['port'], $array[$key]['pw'], $array[$key]['sid'],
				$array[$key]['type'], $array[$key]['user'], $array[$key]['mount']);

			$html .= print_r($data1, true) . '</pre></div>';

			// Second Function
			$html .= '<b>...Parsing data of the first server: </b>
			<button onclick="document.getElementById(\'second\').style.display = \'block\'; return false;">Show Details</button>
			<br /><div id="second" style="display: none;"><pre style="background: #f9f9f9;">';

			$data2 = $ob->parse($key, $data1, $array[$key]['type'], $array[$key]['mount']);

			$html .= print_r($data2, true) . '</pre></div>';

			// Third Function
			$html .= '<b>...Last check run totals function (all servers):</b>
			<button onclick="document.getElementById(\'third\').style.display = \'block\'; return false;">Show Details</button>
			<br /><div id="third" style="display: none;"><pre style="background: #f9f9f9;">';

			$data3 = $ob->totals();

			$html .= print_r($data3, true) . '</pre></div>';

			// Time taken
			$total = round(((microtime(true) - $start) * 1000), 2) / 1000;
			$html .= '<br /><br />Debug process took ' . round($total, 2) . ' second(s) to complete.';

			return $html;

		}


		// Returns single server info (if empty first server in array will be used)
		// IMPORTANT NOTE: If you use this function, array must include ALL details of server.
		public function getInfo($serverID = '') {

			if ($serverID == '') $key = key($this->streams); else $key = $serverID;


			if (!empty($this->streams[$key]['ip']) || !empty($this->streams[$key]['port'])) {

				return ($this->parse($key, $this->getData($this->streams[$key]['ip'], $this->streams[$key]['port'], $this->streams[$key]['pw'], $this->streams[$key]['sid'],
					$this->streams[$key]['type'], $this->streams[$key]['user'], $this->streams[$key]['mount']), $this->streams[$key]['type'], $this->streams[$key]['mount']));

			} else {

				return 'Specified Server doesn\'t exist or required details are missing !';

			}

		}

		// Calculates values from all servers in array and returns them to user
		public function totals() {

			$total = array();

			if ($data = $this->handle()) { // Get arrays from shoutcast

				foreach ($data as $server => $arr) {

					if ($arr == false) { continue; } // Skip count if results ain't correct

					$total['listeners']      += $arr['listeners'];
					$total['peak']           += $arr['peak'];
					$total['listen-time']    += $arr['avg'];
					$total['hits']           += $arr['hits'];

					if (is_array($arr['listeners-list'])) {
						foreach($arr['listeners-list'] as $newarr) {
							$total['listeners-list'][] = $newarr;
						}
					}

					if ($total['status'] == "0") { // If any server before this one is offline, mark all as offline!
						continue;
					} else { // else set status
						$total['status'] = $arr['status'];
					}

				}

				return $total;
				// Return our data and exit script

			} else { // If there is failture in xml2arr just return false fuck it

				return false;

			}

		}

		// This function uses APC cache to speed up loads of requests
		public function cached($name = 'WebRadio') {

			if (!$data = apc_fetch($name)) { // If not cached, create new cache

				$new = $this->totals();
				apc_store($name, $new, 60);
				return $new;

			} else { // If cached, return data.
				return $data;
			}

		}

		private function handle() {

			if(is_array($this->streams)) { // Check first level array for different servers

				foreach ($this->streams as $server => $con) { // Foreach Servers

					if (is_array($con)) { // Check Second level arrays if there is details for the server

						/*   Explanation how do these ifs work:
						*    this will temporary save the variables into memory
						*    so you can specify ip, port, sid, pw, type and mount only once (first array)
						*    this will save time writing details or changing them
						*/
						if (!empty($con['ip']))      { $ip   = $con['ip'];   }
						if (!empty($con['port']))    { $port = $con['port']; }
						if (!empty($con['sid']))     { $sid  = $con['sid'];  }
						if (!empty($con['pw']))      { $pw   = $con['pw'];   }
						if (!empty($con['user']))    { $user = $con['user']; }
						if (!empty($con['type']))    { $type = $con['type']; }
						if (!empty($con['mount']))   { $mount= $con['mount'];}
						// end details

						// Parse servers etc.
						$servers[$server] = $this->parse($server, $this->getData($ip, $port, $pw, $sid, $type, $user, $mount), $type, $mount);

					} else { // Second Level is not array

						echo('ERROR: Second Level array was empty. You must write an array like ie.: (\'ip\' => \'192...\') !');
						return false;

					}

				} // End Foreach

			} else { // First Level is not array?

				echo('ERROR: First Level array was empty, you must specify servers like ie.: array(\'server1\' => array()) !');
				return false;

			} // End IF


			return $servers;

		}

		// New Function to handle Icecast/Shoutcast xml parses (Note: Icecast has to connect twice!)
		private function getData($ip, $port, $pw, $sid = '', $type = 'shoutcast', $user = '', $mount = '') {

			if ($type == 'shoutcast') { // Shoutcast handle

				return self::xml2array($this->connect("http://{$ip}:{$port}/admin.cgi?sid={$sid}&pass={$pw}&mode=viewxml"));

			} elseif ($type == 'icecast') { // Icecast handle

				$serverDetails = $this->connect("http://{$ip}:{$port}/admin/stats", "{$user}:{$pw}");
				$listenDetails = $this->connect("http://{$ip}:{$port}/admin/listclients?mount=/{$mount}", "{$user}:{$pw}");

				if (preg_match("/You need to authenticate/s", $serverDetails)) {

					// Log here if authorization fails
					echo 'Authorization failed!';
					return false;

				} else {

					return array_merge_recursive(self::xml2array($serverDetails), self::xml2array($listenDetails));

				}

			} else { // Type not icecast/shoutcast, return false.

				return false;

			}

		}

		// Simple connect function, returns error if remote connections are forbidden on server.
		private function connect($openURL, $auth = '') {
          // echo $openURL."<br>".$auth;
			if(function_exists('curl_version')) { // if CURL is enabled

				$cUrl = curl_init();
				curl_setopt($cUrl, CURLOPT_URL, $openURL);
				curl_setopt($cUrl, CURLOPT_RETURNTRANSFER, 1 );
				curl_setopt($cUrl, CURLOPT_USERAGENT, "Mozilla");

				if (!empty($auth)) {
					curl_setopt($cUrl, CURLOPT_USERPWD, $auth);
				}

				$data = curl_exec($cUrl);
				curl_close($cUrl);

			} else { // Slower but works on CURL disabled servers

				if (!empty($auth)) {
					$a = stream_context_create(array('http' => array('header'  => "Authorization: Basic " . base64_encode($auth))));
				}

				$data = file_get_contents($openURL, $false, $a);

			}

			return utf8_encode($data);

		}

		// Function to parse Shoutcast info from XML file and fixes listeners array for easier use.
		public function parse($streamName, $arr, $type = 'shoutcast', $mount = '') {

			/****************************************************************************************************
			*   Remake SHOUTCAST array to match both servers array details (used for totals also)
			*****************************************************************************************************/
			if ($type == 'shoutcast') {

				$test = $arr['LISTENERS']['LISTENER'];

				// First Part Fixes Listeners !
				if (is_array($test)) { // Check if there is listeners
					unset($arr['LISTENERS']['LISTENER']);
					if ($test['0'] != "") {

						$arr['LISTENERS'] = $test;

					} else {

						$arr['LISTENERS']['0'] = $test;

					}

					$listeners = count($arr['LISTENERS']);
					for ($i = 0; $i <= $listeners-1; $i++) { // Add StreamName to each listener
						$arr['LISTENERS'][$i]['SERVER'] = $streamName;
					}

				}


				// Now the second part, we create a new array from old one, with details we need!
				$new = array();
				$new['listeners']        = $arr['CURRENTLISTENERS'];
				$new['peak']             = $arr['PEAKLISTENERS'];
				$new['avg']              = $arr['AVERAGETIME'];
				$new['genero']           =$arr['SERVERGENRE'];
				$new['server']           =$arr['SERVERURL'];
				$new['streamtitle']      = $arr['SERVERTITLE'];
				$new['songtitle']        = $arr['SONGTITLE'];
				$new['hits']             = $arr['STREAMHITS'];
				$new['status']           = $arr['STREAMSTATUS'];
				$new['path']             = $arr['STREAMPATH'];
				$new['bitrate']          = $arr['BITRATE'];
				$new['listeners-list']   = $arr['LISTENERS'];


				// Shoutcast specific (23.2.2014) - song history
				if (is_array($arr['SONGHISTORY']['SONG'])) {
					foreach ($arr['SONGHISTORY'] as $newdata) {
						$new['history'][$newdata['PLAYEDAT']] = $newdata['TITLE'];
					}

				}
				/****************************************************************************************************
				*   Remake ICECAST array to match both servers array details (used for totals also)
				*****************************************************************************************************/
			} elseif ($type == 'icecast') {

				$test = $arr['source']['listener'];

				// Fix Listeners arrays !
				if (is_array($test)) { // Check if there is listeners
					unset($arr['source']['listener']);
					if ($test['0'] != "") {

						$arr['LISTENERS'] = $test;

					} else {

						$arr['LISTENERS']['0'] = $test;

					}
				} else { $arr['LISTENERS'] = array(); }

				// Change array keys of listeners to match shoutcast output
				$listeners = count($arr['LISTENERS']);
				for ($i = 0; $i <= $listeners-1; $i++) {

					// Fix for Icecast KH8+
					if (is_array($arr['LISTENERS'][$i]['@attributes'])) {
						unset($arr['LISTENERS'][$i]['@attributes']);
					}

					// Icecast KH8+ Lag parse
					if (isset($arr['LISTENERS'][$i]['lag'])) {
						$arr['LISTENERS'][$i]['LAG'] = $arr['LISTENERS'][$i]['lag'];
						unset($arr['LISTENERS'][$i]['lag']);
					}

					// Rest of parse
					$arr['LISTENERS'][$i]['HOSTNAME']    = $arr['LISTENERS'][$i]['IP'];
					$arr['LISTENERS'][$i]['USERAGENT']   = $arr['LISTENERS'][$i]['UserAgent'];
					$arr['LISTENERS'][$i]['CONNECTTIME'] = $arr['LISTENERS'][$i]['Connected'];
					$arr['LISTENERS'][$i]['UID']         = $arr['LISTENERS'][$i]['ID'];
					$arr['LISTENERS'][$i]['SERVER']      = $streamName;

					unset($arr['LISTENERS'][$i]['IP'], $arr['LISTENERS'][$i]['UserAgent'], $arr['LISTENERS'][$i]['Connected'], $arr['LISTENERS'][$i]['ID']);
				}

				// Now the third part, we create a new array from old one, with details we need!
				$new = array();

				if (empty($arr['source'][1]) AND !empty($mount)) { // Fix for multiple Icecast streams (New 1.01)

					// Added fix for listeners because Kh8+ has array insetead of string (WTF DEVS???)
					$new['listeners']        = ((is_array($arr['source']['listeners'])) ? $arr['source']['listeners'][0] : $arr['source']['listeners']);

					$new['peak']             = $arr['source']['listener_peak'];
					$new['bitrate']          = ($arr['source']['audio_bitrate'] / 1000);
					$new['songtitle']        = $arr['source']['artist'] . $arr['source']['title'];
					$new['path']             = '/' . preg_replace('/http:\/\/(.*)?\//s', '', $arr['source']['listenurl']);

				} else {

					foreach ($arr['source'] as $key => $data) {

						if (isset($data["@attributes"]["mount"]) && $data["@attributes"]["mount"] == "/{$mount}") {
							$new['listeners']        = $arr['source'][$key]['listeners'];
							$new['peak']             = $arr['source'][$key]['listener_peak'];
							$new['bitrate']          = ($arr['source'][$key]['audio_bitrate'] / 1000);
							$new['songtitle']        = $arr['source'][$key]['artist'] . ' - ' . $arr['source'][$key]['title'];
							$new['path']             = '/' . preg_replace('/http:\/\/(.*)?\//s', '', $arr['source'][$key]['listenurl']);
						}
					}

				}

				$new['avg']              = '0';
				$new['hits']             = $arr['listener_connections'];
				$new['status']           = (($arr['0'] == 'Source does not exist' OR $arr['0'] == '<b>mount does not exist</b>') ? '0' : '1'); // Another newer Icecast fix...
				$new['listeners-list']   = $arr['LISTENERS'];


				// Unknown server
			} else {
				return false;
			}

			// Before the output of this function we check if all went ok so far
			if ($new['listeners'] >= '0') { // If listeners is not empty, means all went well

				return $new;

			} else { // Oops listeners not defined, error.

				return false;

			}

		}

		// Very simple trick using json to ease up XML -> array with named keys instead of numbers.
		static function xml2array($data, $lower = false) {

			if ($lower != false) {
				$vals = json_decode(strtolower(json_encode((array)simplexml_load_string($data))),true);
			} else {
				$vals = json_decode(json_encode((array)simplexml_load_string($data)),true);
			}

			return $vals;

		}


	}
?>